@@ -31,10 +31,6 @@
   (:generator 1
     (storew value object offset lowtag)))
 
-(define-vop (init-slot set-slot)
-  (:info name dx-p offset lowtag)
-  (:ignore name dx-p))
-
 (define-vop (compare-and-swap-slot)
   (:args (object :scs (descriptor-reg))
          (old :scs (descriptor-reg any-reg))
@@ -44,17 +40,22 @@
   (:temporary (:sc interior-reg) lip)
   (:results (result :scs (descriptor-reg any-reg) :from :load))
   (:generator 5
-    (inst dsb)
     (inst add-sub lip object (- (* offset n-word-bytes) lowtag))
-    LOOP
-    (inst ldxr result lip)
-    (inst cmp result old)
-    (inst b :ne EXIT)
-    (inst stlxr tmp-tn new lip)
-    (inst cbnz tmp-tn LOOP)
-    EXIT
-    (inst clrex)
-    (inst dmb)))
+    (cond ((member :arm-v8.1 *backend-subfeatures*)
+           (move result old)
+           (inst casal result new lip))
+          (t
+           (assemble ()
+             (inst dsb)
+             LOOP
+             (inst ldxr result lip)
+             (inst cmp result old)
+             (inst b :ne EXIT)
+             (inst stlxr tmp-tn new lip)
+             (inst cbnz tmp-tn LOOP)
+             EXIT
+             (inst clrex)
+             (inst dmb))))))
 
 ;;;; Symbol hacking VOPs:
 
@@ -95,18 +96,42 @@
 
   (define-vop (symbol-value checked-cell-ref)
     (:translate symeval)
+    (:args (symbol :scs (descriptor-reg) :to :save
+                   :load-if (not (and (sc-is symbol constant)
+                                      (or (symbol-always-has-tls-value-p (tn-value symbol))
+                                          (symbol-always-has-tls-index-p (tn-value symbol)))))))
+    (:args-var symbol-tn-ref)
     (:temporary (:sc any-reg) tls-index)
     (:variant-vars check-boundp)
     (:variant t)
     (:generator 9
-      (inst ldr (32-bit-reg tls-index) (tls-index-of object))
-      (inst ldr value (@ thread-tn tls-index))
-      (inst cmp value no-tls-value-marker-widetag)
-      (inst b :ne LOCAL)
-      (loadw value object symbol-value-slot other-pointer-lowtag)
-      LOCAL
+      (let* ((known-symbol-p (sc-is symbol constant))
+             (known-symbol (and known-symbol-p (tn-value symbol)))
+             (fixup (and known-symbol
+                         (make-fixup known-symbol :symbol-tls-index))))
+        (cond
+          ((symbol-always-has-tls-value-p known-symbol)
+           (inst ldr value (@ thread-tn fixup)))
+          (t
+           (cond
+             (known-symbol              ; e.g. CL:*PRINT-BASE*
+              ;; Known nonzero TLS index, but possibly no per-thread value.
+              ;; The TLS value and global value can be loaded independently.
+              (inst ldr value (@ thread-tn fixup)))
+             (t
+              (inst ldr (32-bit-reg tls-index) (tls-index-of symbol))
+              (inst ldr value (@ thread-tn tls-index))))
+
+           (assemble ()
+             (inst cmp value no-tls-value-marker-widetag)
+             (inst b :ne LOCAL)
+             (when known-symbol
+               (load-constant vop symbol (setf symbol (tn-ref-load-tn symbol-tn-ref))))
+             (loadw value symbol symbol-value-slot other-pointer-lowtag)
+             LOCAL))))
+
       (when check-boundp
-        (let ((err-lab (generate-error-code vop 'unbound-symbol-error object)))
+        (let ((err-lab (generate-error-code vop 'unbound-symbol-error symbol)))
           (inst cmp value unbound-marker-widetag)
           (inst b :eq err-lab)))))
 
@@ -228,6 +253,40 @@
     (inst dmb)
     (inst cmp result unbound-marker-widetag)
     (inst b :eq (generate-error-code vop 'unbound-symbol-error symbol))))
+
+(define-vop (%compare-and-swap-symbol-value-v8.1)
+  (:translate %compare-and-swap-symbol-value)
+  (:args (symbol :scs (descriptor-reg))
+         (old :scs (descriptor-reg any-reg))
+         (new :scs (descriptor-reg any-reg)))
+  (:results (result :scs (descriptor-reg any-reg) :from :load))
+  #+sb-thread
+  (:temporary (:sc any-reg) tls-index)
+  (:temporary (:sc interior-reg) lip)
+  (:policy :fast-safe)
+  (:vop-var vop)
+  (:guard (member :arm-v8.1 *backend-subfeatures*))
+  (:generator 14
+    #+sb-thread
+    (assemble ()
+      (inst ldr (32-bit-reg tls-index) (tls-index-of symbol))
+      ;; Thread-local area, no synchronization needed.
+      (inst ldr result (@ thread-tn tls-index))
+      (inst cmp result old)
+      (inst b :ne DONT-STORE-TLS)
+      (inst str new (@ thread-tn tls-index))
+      DONT-STORE-TLS
+
+      (inst cmp result no-tls-value-marker-widetag)
+      (inst b :ne CHECK-UNBOUND))
+    (inst add-sub lip symbol (- (* symbol-value-slot n-word-bytes)
+                                other-pointer-lowtag))
+    (move result old)
+    (inst casal result new lip)
+    CHECK-UNBOUND
+    (inst cmp result unbound-marker-widetag)
+    (inst b :eq (generate-error-code vop 'unbound-symbol-error symbol))))
+
 
 ;;;; Fdefinition (fdefn) objects.
 
@@ -293,7 +352,12 @@
 (progn
   (define-vop (dynbind)
     (:args (value :scs (any-reg descriptor-reg) :to :save)
-           (symbol :scs (descriptor-reg)))
+           (symbol :scs (descriptor-reg)
+                   :load-if (not (and (sc-is symbol constant)
+                                      (or (symbol-always-has-tls-value-p (tn-value symbol))
+                                           (symbol-always-has-tls-index-p (tn-value symbol))
+                                          )))
+                   ))
     (:temporary (:sc descriptor-reg) value-temp)
     (:temporary (:sc descriptor-reg :offset r0-offset :from (:argument 1)) alloc-tls-symbol)
     (:temporary (:sc non-descriptor-reg :offset nl0-offset) tls-index)
@@ -303,19 +367,29 @@
     (:temporary (:scs (any-reg)) bsp)
      (:generator 5
       (load-binding-stack-pointer bsp)
-      (inst ldr (32-bit-reg tls-index) (tls-index-of symbol))
       (inst add bsp bsp (* binding-size n-word-bytes))
       (store-binding-stack-pointer bsp)
-      (inst cbnz (32-bit-reg tls-index) TLS-INDEX-VALID)
-      (move alloc-tls-symbol symbol)
-      (load-inline-constant value-temp '(:fixup alloc-tls-index :assembly-routine) lip)
-      (inst blr value-temp)
-      TLS-INDEX-VALID
-
-      (inst ldr value-temp (@ thread-tn tls-index))
-      (inst stp value-temp tls-index (@ bsp (* (- binding-value-slot binding-size)
-                                               n-word-bytes)))
-      (inst str value (@ thread-tn tls-index))))
+      (let* ((known-symbol-p (sc-is symbol constant))
+             (known-symbol (and known-symbol-p (tn-value symbol)))
+             (tls-index-reg tls-index)
+             (tls-index (if known-symbol
+                            (make-fixup known-symbol :symbol-tls-index)
+                            tls-index)))
+        (assemble ()
+          (cond (known-symbol
+                 (inst movz tls-index-reg tls-index))
+                (t
+                 (inst ldr (32-bit-reg tls-index) (tls-index-of symbol))
+                 (inst cbnz (32-bit-reg tls-index) TLS-INDEX-VALID)
+                 (move alloc-tls-symbol symbol)
+                 (load-inline-constant value-temp '(:fixup alloc-tls-index :assembly-routine) lip)
+                 (inst blr value-temp)))
+          TLS-INDEX-VALID
+          (inst ldr value-temp (@ thread-tn tls-index))
+          (inst stp value-temp tls-index-reg
+                (@ bsp (* (- binding-value-slot binding-size)
+                          n-word-bytes)))
+          (inst str value (@ thread-tn tls-index))))))
 
   (define-vop (unbind-n)
     (:info symbols)
@@ -482,6 +556,7 @@
 (define-full-reffer code-header-ref * 0 other-pointer-lowtag
   (descriptor-reg any-reg) * code-header-ref)
 
+#-darwin-jit
 (define-full-setter code-header-set * 0 other-pointer-lowtag
   (descriptor-reg any-reg) * code-header-set)
 
@@ -507,11 +582,9 @@
                        (inst ,instruction value (@ object offset))))
                     ,@(when move-result
                         `((,move-macro result value))))))
-         (let ((ref-vop (symbolicate "RAW-INSTANCE-REF/" name))
-               (set-vop (symbolicate "RAW-INSTANCE-SET/" name)))
            `(progn
-              (define-vop (,ref-vop)
-                (:translate ,(symbolicate "%" ref-vop))
+              (define-vop ()
+                (:translate ,(symbolicate "%RAW-INSTANCE-REF/" name))
                 (:policy :fast-safe)
                 (:args (object :scs (descriptor-reg))
                        (index :scs (any-reg immediate)))
@@ -520,17 +593,15 @@
                 (:result-types ,value-primtype)
                 (:temporary (:scs (non-descriptor-reg)) offset)
                 (:generator 5 ,@(emit-generator 'ldr nil)))
-              (define-vop (,set-vop)
-                (:translate ,(symbolicate "%" set-vop))
+              (define-vop ()
+                (:translate ,(symbolicate "%RAW-INSTANCE-SET/" name))
                 (:policy :fast-safe)
                 (:args (object :scs (descriptor-reg))
                        (index :scs (any-reg immediate))
-                       (value :scs (,value-sc) :target result))
+                       (value :scs (,value-sc)))
                 (:arg-types * positive-fixnum ,value-primtype)
-                (:results (result :scs (,value-sc)))
-                (:result-types ,value-primtype)
                 (:temporary (:scs (non-descriptor-reg)) offset)
-                (:generator 5 ,@(emit-generator 'str t))))))))
+                (:generator 5 ,@(emit-generator 'str nil)))))))
   (define-raw-slot-vops word unsigned-num unsigned-reg)
   (define-raw-slot-vops signed-word signed-num signed-reg)
   (define-raw-slot-vops single single-float single-reg
@@ -566,3 +637,22 @@
     (inst stlxr tmp-tn sum lip)
     (inst cbnz tmp-tn LOOP)
     (inst dmb)))
+
+(define-vop (raw-instance-atomic-incf/word-v8.1)
+  (:translate %raw-instance-atomic-incf/word)
+  (:policy :fast-safe)
+  (:args (object :scs (descriptor-reg))
+         (index :scs (any-reg))
+         (diff :scs (unsigned-reg)))
+  (:arg-types * positive-fixnum unsigned-num)
+  (:temporary (:sc interior-reg) lip)
+  (:results (result :scs (unsigned-reg) :from :load))
+  (:result-types unsigned-num)
+  (:guard (member :arm-v8.1 *backend-subfeatures*))
+  (:generator 3
+    (inst add lip object (lsl index (- word-shift n-fixnum-tag-bits)))
+    (inst add lip lip (- (* instance-slots-offset
+                            n-word-bytes)
+                         instance-pointer-lowtag))
+
+    (inst ldaddal diff result lip)))
