@@ -47,10 +47,6 @@
   (:generator 1
     (storew value object offset lowtag)))
 
-(define-vop (init-slot set-slot)
-  (:info name dx-p offset lowtag)
-  (:ignore name dx-p))
-
 (define-vop (compare-and-swap-slot)
   (:args (object :scs (descriptor-reg))
          (old :scs (descriptor-reg any-reg))
@@ -486,7 +482,7 @@
   (:variant funcallable-instance-info-offset fun-pointer-lowtag)
   (:translate %funcallable-instance-info))
 
-(define-vop (set-funcallable-instance-info word-index-set)
+(define-vop (set-funcallable-instance-info word-index-set-nr)
   (:variant funcallable-instance-info-offset fun-pointer-lowtag)
   (:translate %set-funcallable-instance-info))
 
@@ -598,7 +594,7 @@
         (%primitive code-header-ref-fdefn code index)
         (%primitive code-header-ref-any code index))))
 
-(define-vop (code-header-set word-index-set)
+(define-vop (code-header-set word-index-set-nr)
   (:translate code-header-set)
   (:policy :fast-safe)
   (:variant 0 other-pointer-lowtag))
@@ -614,30 +610,19 @@
 
 (macrolet ((def (suffix sc primtype)
              `(progn
-                (define-vop (,(symbolicate "RAW-INSTANCE-INIT/" suffix))
-                  (:args (object :scs (descriptor-reg))
-                         (value :scs (,sc)))
-                  (:arg-types * ,primtype)
-                  (:info index)
-                  (:temporary (:scs (non-descriptor-reg)) temp)
-                  (:generator 4
-                    (inst lr temp (offset-for-raw-slot index))
-                    (inst stdx value object temp)))
-                (define-vop (,(symbolicate "RAW-INSTANCE-REF/" suffix) word-index-ref)
+                (define-vop (,(symbolicate "%RAW-INSTANCE-REF/" suffix) word-index-ref)
                   (:policy :fast-safe)
                   (:translate ,(symbolicate "%RAW-INSTANCE-REF/" suffix))
                   (:variant instance-slots-offset instance-pointer-lowtag)
                   (:arg-types instance positive-fixnum)
                   (:results (value :scs (,sc)))
                   (:result-types ,primtype))
-                (define-vop (,(symbolicate "RAW-INSTANCE-SET/" suffix) word-index-set)
+                (define-vop (,(symbolicate "%RAW-INSTANCE-SET/" suffix) word-index-set-nr)
                   (:policy :fast-safe)
                   (:translate ,(symbolicate "%RAW-INSTANCE-SET/" suffix))
                   (:variant instance-slots-offset instance-pointer-lowtag)
                   (:arg-types instance positive-fixnum ,primtype)
-                  (:args (object) (index) (value :scs (,sc)))
-                  (:results (result :scs (,sc)))
-                  (:result-types ,primtype)))))
+                  (:args (object) (index) (value :scs (,sc)))))))
   (def word unsigned-reg unsigned-num)
   (def signed-word signed-reg signed-num))
 
@@ -667,32 +652,7 @@
     (inst bne LOOP)
     (inst isync)))
 
-(define-vop (raw-instance-ref/word word-index-ref)
-  (:policy :fast-safe)
-  (:translate %raw-instance-ref/word)
-  (:variant instance-slots-offset instance-pointer-lowtag)
-  (:arg-types instance positive-fixnum)
-  (:results (value :scs (unsigned-reg)))
-  (:result-types unsigned-num))
-
-(define-vop (raw-instance-set/word word-index-set)
-  (:policy :fast-safe)
-  (:translate %raw-instance-set/word)
-  (:variant instance-slots-offset instance-pointer-lowtag)
-  (:arg-types instance positive-fixnum unsigned-num)
-  (:args (object) (index) (value :scs (unsigned-reg)))
-  (:results (result :scs (unsigned-reg)))
-  (:result-types unsigned-num))
-
-(define-vop (raw-instance-init/single)
-  (:args (object :scs (descriptor-reg))
-         (value :scs (single-reg)))
-  (:arg-types * single-float)
-  (:info index)
-  (:generator 4
-    (inst stfs value object (offset-for-raw-slot index))))
-
-(define-vop (raw-instance-ref/single)
+(define-vop ()
   (:translate %raw-instance-ref/single)
   (:policy :fast-safe)
   (:args (object :scs (descriptor-reg))
@@ -707,33 +667,21 @@
                                instance-pointer-lowtag))
     (inst lfsx value object offset)))
 
-(define-vop (raw-instance-set/single)
+(define-vop ()
   (:translate %raw-instance-set/single)
   (:policy :fast-safe)
   (:args (object :scs (descriptor-reg))
          (index :scs (any-reg))
-         (value :scs (single-reg) :target result))
+         (value :scs (single-reg)))
   (:arg-types * positive-fixnum single-float)
-  (:results (result :scs (single-reg)))
-  (:result-types single-float)
   (:temporary (:scs (non-descriptor-reg)) offset)
   (:generator 5
     (inst sldi offset index (- word-shift n-fixnum-tag-bits))
     (inst addi offset offset (- (ash instance-slots-offset word-shift)
                                instance-pointer-lowtag))
-    (inst stfsx value object offset)
-    (unless (location= result value)
-      (inst frsp result value))))
+    (inst stfsx value object offset)))
 
-(define-vop (raw-instance-init/double)
-  (:args (object :scs (descriptor-reg))
-         (value :scs (double-reg)))
-  (:arg-types * double-float)
-  (:info index)
-  (:generator 4
-    (inst stfd value object (offset-for-raw-slot index))))
-
-(define-vop (raw-instance-ref/double)
+(define-vop ()
   (:translate %raw-instance-ref/double)
   (:policy :fast-safe)
   (:args (object :scs (descriptor-reg))
@@ -748,36 +696,21 @@
                                instance-pointer-lowtag))
     (inst lfdx value object offset)))
 
-(define-vop (raw-instance-set/double)
+(define-vop ()
   (:translate %raw-instance-set/double)
   (:policy :fast-safe)
   (:args (object :scs (descriptor-reg))
          (index :scs (any-reg))
-         (value :scs (double-reg) :target result))
+         (value :scs (double-reg)))
   (:arg-types * positive-fixnum double-float)
-  (:results (result :scs (double-reg)))
-  (:result-types double-float)
   (:temporary (:scs (non-descriptor-reg)) offset)
   (:generator 5
     (inst sldi offset index (- word-shift n-fixnum-tag-bits))
     (inst addi offset offset (- (ash instance-slots-offset word-shift)
                                instance-pointer-lowtag))
-    (inst stfdx value object offset)
-    (unless (location= result value)
-      (inst fmr result value))))
+    (inst stfdx value object offset)))
 
-(define-vop (raw-instance-init/complex-single)
-  (:args (object :scs (descriptor-reg))
-         (value :scs (complex-single-reg)))
-  (:arg-types * complex-single-float)
-  (:info index)
-  (:generator 4
-    (inst stfs (complex-single-reg-real-tn value)
-          object (offset-for-raw-slot index))
-    (inst stfs (complex-single-reg-imag-tn value)
-          object (offset-for-raw-slot index (/ n-word-bytes 2)))))
-
-(define-vop (raw-instance-ref/complex-single)
+(define-vop ()
   (:translate %raw-instance-ref/complex-single)
   (:policy :fast-safe)
   (:args (object :scs (descriptor-reg))
@@ -794,44 +727,23 @@
     (inst addi offset offset (/ n-word-bytes 2))
     (inst lfsx (complex-single-reg-imag-tn value) object offset)))
 
-(define-vop (raw-instance-set/complex-single)
+(define-vop ()
   (:translate %raw-instance-set/complex-single)
   (:policy :fast-safe)
   (:args (object :scs (descriptor-reg))
          (index :scs (any-reg))
-         (value :scs (complex-single-reg) :target result))
+         (value :scs (complex-single-reg)))
   (:arg-types * positive-fixnum complex-single-float)
-  (:results (result :scs (complex-single-reg)))
-  (:result-types complex-single-float)
   (:temporary (:scs (non-descriptor-reg)) offset)
   (:generator 5
     (inst sldi offset index (- word-shift n-fixnum-tag-bits))
     (inst addi offset offset (- (ash instance-slots-offset word-shift)
                                instance-pointer-lowtag))
-    (let ((value-real (complex-single-reg-real-tn value))
-          (result-real (complex-single-reg-real-tn result)))
-      (inst stfsx value-real object offset)
-      (unless (location= result-real value-real)
-        (inst frsp result-real value-real)))
+    (inst stfsx (complex-single-reg-real-tn value) object offset)
     (inst addi offset offset (/ n-word-bytes 2))
-    (let ((value-imag (complex-single-reg-imag-tn value))
-          (result-imag (complex-single-reg-imag-tn result)))
-      (inst stfsx value-imag object offset)
-      (unless (location= result-imag value-imag)
-        (inst frsp result-imag value-imag)))))
+    (inst stfsx (complex-single-reg-imag-tn value) object offset)))
 
-(define-vop (raw-instance-init/complex-double)
-  (:args (object :scs (descriptor-reg))
-         (value :scs (complex-double-reg)))
-  (:arg-types * complex-double-float)
-  (:info index)
-  (:generator 4
-    (inst stfd (complex-single-reg-real-tn value)
-          object (offset-for-raw-slot index))
-    (inst stfd (complex-double-reg-imag-tn value)
-          object (offset-for-raw-slot index n-word-bytes))))
-
-(define-vop (raw-instance-ref/complex-double)
+(define-vop ()
   (:translate %raw-instance-ref/complex-double)
   (:policy :fast-safe)
   (:args (object :scs (descriptor-reg))
@@ -848,28 +760,18 @@
     (inst addi offset offset n-word-bytes)
     (inst lfdx (complex-double-reg-imag-tn value) object offset)))
 
-(define-vop (raw-instance-set/complex-double)
+(define-vop ()
   (:translate %raw-instance-set/complex-double)
   (:policy :fast-safe)
   (:args (object :scs (descriptor-reg))
          (index :scs (any-reg))
-         (value :scs (complex-double-reg) :target result))
+         (value :scs (complex-double-reg)))
   (:arg-types * positive-fixnum complex-double-float)
-  (:results (result :scs (complex-double-reg)))
-  (:result-types complex-double-float)
   (:temporary (:scs (non-descriptor-reg)) offset)
   (:generator 5
     (inst sldi offset index (- word-shift n-fixnum-tag-bits))
     (inst addi offset offset (- (ash instance-slots-offset word-shift)
                                instance-pointer-lowtag))
-    (let ((value-real (complex-double-reg-real-tn value))
-          (result-real (complex-double-reg-real-tn result)))
-      (inst stfdx value-real object offset)
-      (unless (location= result-real value-real)
-        (inst fmr result-real value-real)))
+    (inst stfdx (complex-double-reg-real-tn value) object offset)
     (inst addi offset offset n-word-bytes)
-    (let ((value-imag (complex-double-reg-imag-tn value))
-          (result-imag (complex-double-reg-imag-tn result)))
-      (inst stfdx value-imag object offset)
-      (unless (location= result-imag value-imag)
-        (inst fmr result-imag value-imag)))))
+    (inst stfdx (complex-double-reg-imag-tn value) object offset)))
