@@ -1,16 +1,5 @@
 (in-package "SB-X86-64-ASM")
 
-(defun get-avx2 (number)
-  (svref (load-time-value
-          (coerce (loop for i from 0 below 16
-                        collect (!make-reg (!make-avx2-id i)))
-                  'vector)
-          t)
-         number))
-
-(defun is-avx2-id-p (reg-id)
-  (= (ldb (byte 3 0) reg-id) 3))
-
 (define-arg-type ymmreg
   :prefilter #'prefilter-reg-r
   :printer #'print-ymmreg)
@@ -228,7 +217,7 @@
                0
                1))
          (xmm-size (r)
-           (cond ((is-avx2-id-p (reg-id r))
+           (cond ((is-ymm-id-p (reg-id r))
                   1)
                  ((xmm-register-p r)
                   0))))
@@ -581,6 +570,31 @@
   (def vpmovzxwq #x66 #x34 #x0f38)
   (def vpmovzxdq #x66 #x35 #x0f38))
 
+(macrolet ((def (name prefix)
+             `(define-instruction ,name (segment dst src pattern)
+                ,@(avx2-inst-printer-list
+                   'ymm-ymm/mem-imm prefix #x70
+                   :printer '(:name :tab reg ", " reg/mem ", " imm))
+                (:emitter
+                 (emit-avx2-inst segment dst src ,prefix #x70
+                                 :remaining-bytes 1)
+                 (emit-byte segment pattern)))))
+  (def vpshufd  #x66)
+  (def vpshufhw #xf3)
+  (def vpshuflw #xf2))
+
+(macrolet ((def (name prefix)
+             `(define-instruction ,name (segment dst src src2 pattern)
+                ,@(avx2-inst-printer-list
+                   'ymm-ymm/mem-imm prefix #xc6)
+                (:emitter
+                 (emit-avx2-inst segment src2 dst ,prefix #xc6
+                                 :vvvv src
+                                 :remaining-bytes 1)
+                 (emit-byte segment pattern)))))
+  (def vshufpd #x66)
+  (def vshufps nil))
+
 (macrolet
     ((def (name prefix opcode)
        `(define-instruction ,name (segment dst src src2 imm)
@@ -1096,11 +1110,11 @@
                                  :w ,w
                                  :l ,(ecase sizing
                                        ((xmm/ymm-vmx/y xmm/ymm-vmx)
-                                        `(if (is-avx2-id-p (reg-id dst))
+                                        `(if (is-ymm-id-p (reg-id dst))
                                              1
                                              0))
                                        (xmm-vmx/y
-                                        `(if (eq (sc-name (tn-sc (ea-index vm))) 'avx2-reg)
+                                        `(if (eq (sc-name (tn-sc (ea-index vm))) 'ymm-reg)
                                              1
                                              0)))
                                  :vm t)))))
