@@ -52,11 +52,9 @@
                `(defmethod ,name ,args
                  (declare (ignore initargs))
                  (error 'metaobject-initialization-violation
-                  ;; FIXME: I'm pretty sure this wants to be "~~@<~A~~@:>"
-                  :format-control ,(coerce (format nil "~@<~A~@:>" control)
-                                           'base-string)
-                  :format-arguments (list ',name)
-                  :references '((:amop :initialization method))))))
+                        :format-control ,(format nil "~~@<~A~~@:>" control)
+                        :format-arguments (list ',name)
+                        :references '((:amop :initialization method))))))
   (def reinitialize-instance ((method method) &rest initargs)
     "Method objects cannot be redefined by ~S.")
   (def change-class ((method method) new &rest initargs)
@@ -97,6 +95,16 @@
     (invalid-method-initarg method "~@<~S of ~S is not a ~S.~@:>"
                             :function fun 'function)))
 
+(macrolet ((dolist-carefully ((var list improper-list-handler) &body body)
+             `(let ((,var nil)
+                    (.dolist-carefully. ,list))
+                (loop (when (null .dolist-carefully.) (return nil))
+                   (if (consp .dolist-carefully.)
+                       (progn
+                         (setq ,var (pop .dolist-carefully.))
+                         ,@body)
+                       (,improper-list-handler))))))
+
 (defun check-qualifiers (method qualifiers)
   (flet ((improper-list ()
            (invalid-method-initarg method
@@ -137,6 +145,7 @@
                      ~V[~;~1{~S~}~;~1{~S and ~S~}~:;~{~#[~;and ~]~S~^, ~}~] ~
                      as ~2:*~V[~;a specializer~:;specializers~].~@:>"
                     (length frcs) frcs)))))
+) ; end MACROLET
 
 (defmethod shared-initialize :before
     ((method standard-method) slot-names &key
@@ -167,8 +176,6 @@
   (declare (ignore slot-names method-cell))
   (initialize-method-function initargs method))
 
-(define-load-time-global *the-class-generic-function*
-  (find-class 'generic-function))
 (define-load-time-global *the-class-standard-generic-function*
   (find-class 'standard-generic-function))
 
@@ -249,36 +256,37 @@
 
 (defun real-get-method (generic-function qualifiers specializers
                         &optional (errorp t)
-                        always-check-specializers)
-  (let ((specializer-count (length specializers))
-        (methods (generic-function-methods generic-function)))
-    (when (or methods always-check-specializers)
-      (let ((required-parameter-count
-             (length (arg-info-metatypes (gf-arg-info generic-function)))))
-        ;; Since we internally bypass FIND-METHOD by using GET-METHOD
-        ;; instead we need to do this here or users may get hit by a
-        ;; failed AVER instead of a sensible error message.
-        (unless (= specializer-count required-parameter-count)
-          (error
-           'find-method-length-mismatch
-           :format-control   "~@<The generic function ~S takes ~D ~
+                                  always-check-specializers)
+  (sb-thread::with-recursive-system-lock ((gf-lock generic-function))
+    (let ((specializer-count (length specializers))
+          (methods (generic-function-methods generic-function)))
+      (when (or methods always-check-specializers)
+        (let ((required-parameter-count
+                (length (arg-info-metatypes (gf-arg-info generic-function)))))
+          ;; Since we internally bypass FIND-METHOD by using GET-METHOD
+          ;; instead we need to do this here or users may get hit by a
+          ;; failed AVER instead of a sensible error message.
+          (unless (= specializer-count required-parameter-count)
+            (error
+             'find-method-length-mismatch
+             :format-control   "~@<The generic function ~S takes ~D ~
                               required argument~:P; was asked to ~
                               find a method with specializers ~:S~@:>"
-           :format-arguments (list generic-function required-parameter-count
-                                   (unparse-specializers generic-function specializers))))))
-    (flet ((congruentp (other-method)
-             (let ((other-specializers (method-specializers other-method)))
-               (aver (= specializer-count (length other-specializers)))
-               (and (equal qualifiers (safe-method-qualifiers other-method))
-                    (every #'same-specializer-p specializers other-specializers)))))
-      (declare (dynamic-extent #'congruentp))
-      (cond ((find-if #'congruentp methods))
-            ((null errorp) nil)
-            (t
-             (error "~@<There is no method on ~S with ~:[no ~
+             :format-arguments (list generic-function required-parameter-count
+                                     (unparse-specializers generic-function specializers))))))
+      (flet ((congruentp (other-method)
+               (let ((other-specializers (method-specializers other-method)))
+                 (aver (= specializer-count (length other-specializers)))
+                 (and (equal qualifiers (safe-method-qualifiers other-method))
+                      (every #'same-specializer-p specializers other-specializers)))))
+        (declare (dynamic-extent #'congruentp))
+        (cond ((find-if #'congruentp methods))
+              ((null errorp) nil)
+              (t
+               (error "~@<There is no method on ~S with ~:[no ~
                      qualifiers~;~:*qualifiers ~:S~] and specializers ~
                      ~:S.~@:>"
-                    generic-function qualifiers specializers))))))
+                      generic-function qualifiers specializers)))))))
 
 (defmethod find-method ((generic-function standard-generic-function)
                         qualifiers specializers &optional (errorp t))
@@ -404,28 +412,36 @@
                       :lambda-list lambda-list
                       :argument-precedence-order argument-precedence-order)
         (set-arg-info gf))
+    (let ((mc (generic-function-method-combination gf)))
+      (setf (gethash gf (method-combination-%generic-functions mc)) t))
     (when (arg-info-valid-p (slot-value gf 'arg-info))
       (update-dfun gf))))
 
 (defmethod reinitialize-instance :around
     ((gf standard-generic-function) &rest args &key
      (lambda-list nil lambda-list-p) (argument-precedence-order nil apo-p))
-  (let ((old-mc (generic-function-method-combination gf)))
+  (let* ((old-mc (generic-function-method-combination gf))
+         (mc (getf args :method-combination old-mc)))
+    (unless (eq mc old-mc)
+      (aver (gethash gf (method-combination-%generic-functions old-mc)))
+      (aver (not (gethash gf (method-combination-%generic-functions mc)))))
     (prog1 (call-next-method)
-      ;; KLUDGE: EQ is too strong a test.
-      (unless (eq old-mc (generic-function-method-combination gf))
+      (unless (eq mc old-mc)
+        (remhash gf (method-combination-%generic-functions old-mc))
+        (setf (gethash gf (method-combination-%generic-functions mc)) t)
         (flush-effective-method-cache gf))
-      (cond
-        ((and lambda-list-p apo-p)
-         (set-arg-info gf
-                       :lambda-list lambda-list
-                       :argument-precedence-order argument-precedence-order))
-        (lambda-list-p (set-arg-info gf :lambda-list lambda-list))
-        (t (set-arg-info gf)))
-      (when (arg-info-valid-p (gf-arg-info gf))
-        (update-dfun gf))
-      (map-dependents gf (lambda (dependent)
-                           (apply #'update-dependent gf dependent args))))))
+      (sb-thread::with-recursive-system-lock ((gf-lock gf))
+        (cond
+          ((and lambda-list-p apo-p)
+           (set-arg-info gf
+                         :lambda-list lambda-list
+                         :argument-precedence-order argument-precedence-order))
+          (lambda-list-p (set-arg-info gf :lambda-list lambda-list))
+          (t (set-arg-info gf)))
+        (when (arg-info-valid-p (gf-arg-info gf))
+          (update-dfun gf))
+        (map-dependents gf (lambda (dependent)
+                             (apply #'update-dependent gf dependent args)))))))
 
 (defun set-methods (gf methods)
   (setf (generic-function-methods gf) nil)
@@ -507,7 +523,7 @@
         ;; don't, however this branch should never be reached because the
         ;; info only stores :GENERIC-FUNCTION when methods are loaded.
         ;; Maybe AVER that it does not happen?
-        (sb-c::ftype-from-fdefn name))))
+        (sb-impl::ftype-from-fdefn name))))
 
 (defun real-add-method (generic-function method &optional skip-dfun-update-p)
   (flet ((similar-lambda-lists-p (old-method new-lambda-list)
@@ -765,7 +781,7 @@
         (compute-applicable-methods-using-types gf types)
       (let ((generator (get-secondary-dispatch-function1
                         gf methods types nil t all-applicable-and-sorted-p)))
-        (make-callable gf methods generator
+        (make-callable generator
                        nil (mapcar #'class-wrapper classes))))))
 
 (defun value-for-caching (gf classes)
@@ -906,9 +922,9 @@
                                    'get-accessor-method-function)))
                     ,optimized-std-fun)))
                 (wrappers
-                 (let ((wrappers (list (layout-of class)
+                 (let ((wrappers (list (wrapper-of class)
                                        (class-wrapper class)
-                                       (layout-of slotd))))
+                                       (wrapper-of slotd))))
                    (if (eq type 'writer)
                        (cons (class-wrapper *the-class-t*) wrappers)
                        wrappers)))
@@ -1078,12 +1094,16 @@
 (defmacro class-test (arg class)
   (cond
     ((eq class *the-class-t*) t)
-    ((eq class *the-class-slot-object*)
-     `(not (typep (classoid-of ,arg) 'system-classoid)))
     ((eq class *the-class-standard-object*)
      `(or (std-instance-p ,arg) (fsc-instance-p ,arg)))
     ((eq class *the-class-funcallable-standard-object*)
      `(fsc-instance-p ,arg))
+    ;; This is going to be cached (in *fgens*),
+    ;; and structure type tests do not check for invalid layout.
+    ;; Cache the wrapper itself, which is going to be different after
+    ;; redifinition.
+    ((structure-class-p class)
+     `(sb-c::%instance-typep ,arg ,(class-wrapper class)))
     (t
      `(typep ,arg ',(class-name class)))))
 
@@ -1262,8 +1282,8 @@
 
 (defun compute-secondary-dispatch-function (generic-function net &optional
                                             method-alist wrappers)
-  (function-funcall (compute-secondary-dispatch-function1 generic-function net)
-                    method-alist wrappers))
+  (funcall (the function (compute-secondary-dispatch-function1 generic-function net))
+           method-alist wrappers))
 
 (defvar *eq-case-table-limit* 15)
 (defvar *case-table-limit* 10)
@@ -1413,17 +1433,17 @@
       (multiple-value-bind (cfunction constants)
           ;; We don't want NAMED-LAMBDA for any expressions handed to FNGEN,
           ;; because name mismatches will render the hashing ineffective.
-          (get-fun1 `(lambda ,arglist
+          (get-fun `(lambda ,arglist
                       (declare (optimize (sb-c::store-closure-debug-pointer 3)))
                       ,@(unless function-p
                           `((declare (ignore .pv. .next-method-call.))))
                       (locally (declare #.*optimize-speed*)
-                               (let ((emf ,net))
-                                 ,(make-emf-call nargs applyp 'emf))))
-                    #'net-test-converter
-                    #'net-code-converter
-                    (lambda (form)
-                      (net-constant-converter form generic-function)))
+                        (let ((emf ,net))
+                          ,(make-emf-call nargs applyp 'emf))))
+                   #'net-test-converter
+                   #'net-code-converter
+                   (lambda (form)
+                     (net-constant-converter form generic-function)))
         (lambda (method-alist wrappers)
           (let* ((alist (list nil))
                  (alist-tail alist))
@@ -1654,7 +1674,7 @@
    gf (generic-function-encapsulations gf) (call-next-method)))
 
 (defmethod (setf class-name) (new-value class)
-  (let ((classoid (layout-classoid (class-wrapper class))))
+  (let ((classoid (wrapper-classoid (class-wrapper class))))
     (if (and new-value (symbolp new-value))
         (setf (classoid-name classoid) new-value)
         (setf (classoid-name classoid) nil)))
